%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is used to calculate electromagnetic field for TE incidence based on RCWA 
% Dr. Zhuomin Zhang's group at Georgia Tech
% Last modified by Bo Zhao (September 2014)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all; 
clc;
tic
time1=fix(clock);                   % start time, [sec]
c0 = 299792458;                   % speed of light in vacuum, [m/s]
e0=8.854187817e-12;         % vacuum permittivity 
mu0=pi*4e-7;                        % vacuum permeability

theta =(0)*pi/180;                % angle of incidence, [rad]
lambda=.484;                        % wavelength in vacuum, [um]
wn=1e4/lambda;                  % wavenumber, [cm-1]
w = 1e6*2*pi*c0./lambda; % angular frequency, [rad/s]
k0 = 2*pi/lambda;               %wavevector
d = [1 0.8 1];                        % thickness of each layer [um] 
N = length(d);                      % # of layers
Period(1:N) =0.4;                 % Period [um]
width =[0 0.2 0];                 % width of metal strips, [um]
psi = width/Period(1);        % filling ratio of each layer
f1 = [0 0.75 0];                     % normalized position for left-end of metal strip
f2 = f1+psi;                           % normalized position for right-end of metal strip
Num_ord = 101;                  % number for the highest diffraction order

%==========================================
     e(1) = 1;   % Usually is air or vacuum 
     % Layered structure
      e_m(1:N) = Palik_Ag(lambda);            % Ridge material (metal)  
      e_d(1:N) = 1+1i*1e-12;                        % Groove material (air)
      %Substrate
      e(2)= 1;                                                    %  air or opaque substrate
      [Ref, Tran,W,V,Cnp,Cnn,Sigma,Kx] = RCWA_Multi_TE(N, e_m, e_d, f1, f2, Period, d, e, lambda, theta, Num_ord); 
%==========================================
% All the matrix are ready.
% Prepare mesh
x=linspace(0,Period(1),100);
layernum=[50 10 50]; % The discritized number for each layer from the incident medium to substrate

for ind=1:N
    z{ind}=linspace(0,d(ind),layernum(ind));
    end
extrad(1)=0;
for indd=2:length(d)
    extrad(indd)=extrad(indd-1)+d(indd-1);
end


for ind=1:N
    W1=W{ind};    
    V1=V{ind};
    for indz=1:length(z{ind})
        zz=z{ind}; % current matrix for z cooridinate 
        Ey=W1*(diag(diag(exp(Sigma{ind}*k0*(zz(indz)-d(ind)))))*Cnp{ind}+diag(diag(exp(-Sigma{ind}*k0*zz(indz))))*Cnn{ind});  
        Hx=1i/c0/mu0*W1*diag(diag(Sigma{ind}))*(diag(diag(exp(Sigma{ind}*k0*(zz(indz)-d(ind)))))*Cnp{ind}-diag(diag(exp(-Sigma{ind}*k0*zz(indz))))*Cnn{ind});
        Hz=1/c0/mu0*(Kx*Ey);  
        Kxcolumn=diag(Kx);
        B=exp(1i*k0*(Kxcolumn*x)); 
        Electy=Ey.'*B;  
        Magnetx=Hx.'*B;
        Magnetz=Hz.'*B;
        Ix(1:length(x),sum(layernum(1:ind-1))+indz)=Magnetx; 
        Iz(1:length(x),sum(layernum(1:ind-1))+indz)=-Magnetz; 
        Mag(1:length(x),sum(layernum(1:ind-1))+indz)=log10(abs(Electy).^2);
        realz(sum(layernum(1:ind-1))+indz)=extrad(N)-(zz(indz)+extrad(ind)); % convert the coordinate for later plotting
    end
end

figure;
pcolor(x,realz,((Mag'))); hold on;
shading interp; colormap(hot); %axis equal;
box on;
set(gca,'linewidth',2,'fontsize',14,'fontname','arial','TickDir','out');
h=colorbar('linewidth',2,'fontsize',14,'fontname','arial','TickDir','out');
xlabel('x (\mum)');
ylabel('z (\mum)');




