%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program is used to calculate electromagnetic field for TM incidence based on RCWA 
% Dr. Zhuomin Zhang's group at Georgia Tech
% Last modified by Bo Zhao (September 2014)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; 
clc;
tic
time1=fix(clock);                   % start time, [sec]
c0 = 299792458;                   % speed of light in vacuum, [m/s]
e0=8.854187817e-12;         % vacuum permittivity 
mu0=pi*4e-7;                        % vacuum permeability

theta =(0)*pi/180;                % angle of incidence, [rad]
lambda=1.95;                        % wavelength in vacuum, [um]
wn=1e4/lambda;                  % wavenumber, [cm-1]
w = 1e6*2*pi*c0./lambda; % angular frequency, [rad/s]
k0 = 2*pi/lambda;               %wavevector
d = [1 0.8 1];                        % thickness of each layer [um] 
N = length(d);                      % # of layers
Period(1:N) =0.4;                 % Period [um]
width =[0 0.2 0];                 % width of metal strips, [um]
psi = width/Period(1);        % filling ratio of each layer
f1 = [0 0.75 0];                     % normalized position for left-end of metal strip
f2 = f1+psi;                           % normalized position for right-end of metal strip
Num_ord = 101;                  % number for the highest diffraction order

%==========================================
     e(1) = 1;   % Usually is air or vacuum 
     % Layered structure
      e_m(1:N) = Palik_Ag(lambda);            % Ridge material (metal) 
      e_d(1:N) = 1+1i*1e-12;                        % Groove material (air)
      %Substrate
      e(2)= 1;      %  air or opaque substrate
      [Ref, Tran,W,V,Cnp,Cnn,E,Sigma,Kx] = RCWA_Multi_TM(N, e_m, e_d, f1, f2, Period, d, e, lambda, theta, Num_ord); 
%==========================================
% All the matrix are ready.
% Prepare mesh
x=linspace(0,Period(1),19);
layernum=[50 30 50]; % The discritized number for each layer from the incident medium to substrate

for ind=1:N
    z{ind}=linspace(0,d(ind),layernum(ind));
    end
extrad(1)=0;
for indd=2:length(d)
    extrad(indd)=extrad(indd-1)+d(indd-1);
end
% The dielectric function of the grating layer
n1=min(find(x>0.25*Period(1))); n2=max(find(x<0.75*Period(1)));
e2=[ones(1,n1-1)*e_m(2) ones(1,n2-n1+1)*e_d(2) ones(1,length(x)-n2)*e_m(2)];

for ind=1:N
    W1=W{ind};    
    V1=V{ind};
    for indz=1:length(z{ind})
        zz=z{ind}; % current matrix for z cooridinate 
        Hy=W1*(diag(diag(exp(Sigma{ind}*k0*(zz(indz)-d(ind)))))*Cnp{ind}+diag(diag(exp(-Sigma{ind}*k0*zz(indz))))*Cnn{ind});  
        Dx=-1i/c0/e0*W1*diag(diag(Sigma{ind}))*(diag(diag(exp(Sigma{ind}*k0*(zz(indz)-d(ind)))))*Cnp{ind}-diag(diag(exp(-Sigma{ind}*k0*zz(indz))))*Cnn{ind});
        Ez=-1/e0/c0*inv(E{ind})*(Kx*Hy);  
        Kxcolumn=diag(Kx);
        B=exp(1i*k0*(Kxcolumn*x)); 
        Magnety=Hy.'*B;  
        Disx=Dx.'*B;
        if ind==2
            Electx=Disx./e2;
        else
            Electx=Disx./e_d(ind);
        end
            Electz=Ez.'*B;
            Magnetz=zeros(1,length(x));
            Magnetx=zeros(1,length(x));
            Electy=zeros(1,length(x));
            MagnitudeH=((abs(Magnety).^2));
            MagnitudeE=((abs(Electy).^2));
            Ix(1:length(x),sum(layernum(1:ind-1))+indz)=Electx; 
            Iz(1:length(x),sum(layernum(1:ind-1))+indz)=-Electz; 
            Mag(1:length(x),sum(layernum(1:ind-1))+indz)=log10(MagnitudeH);
            realz(sum(layernum(1:ind-1))+indz)=extrad(N)-(zz(indz)+extrad(ind)); % convert the coordinate for later plotting
    end
end
figure;
pcolor(x,realz,((Mag')));hold on;
shading interp; colormap(hot);% axis equal;
box on;
set(gca,'linewidth',2,'fontsize',14,'fontname','arial','TickDir','out');
h=colorbar('linewidth',2,'fontsize',14,'fontname','arial','TickDir','out');
xlabel('x (\mum)');
ylabel('z (\mum)');



